%module coral
%{
#include <coral/lib/ref_counted.h>
#include <coral/lib/List.h>
#include <coral/core/python_compat.h>
#include <coral/core/Element.h>
#include <coral/core/ElementDefinition.h>
#include <coral/core/Collection.h>
#include <coral/core/PrimitiveProperties.h>
#include <coral/core/garbage_collector.h>
#include <coral/core/Project.h>
#include <coral/core/PyOperation.h>
#include <coral/core/xmlns_conversion_table.h>
#include <coral/core/util/TransactionManager.h>
#include <coral/core/util/ModelCopy.h>
#include <coral/core/util/SameModel.h>
#include <coral/core/util/uElement.h>
#include <coral/core/diff/Difference.h>
#include <coral/core/io/io.h>
#include <coral/core/CorePrimitives.h>
#include <coral/lib/debugstream.h>
#include <coral/lib/helper_uuid.h>
#include <coral/lib/unicodestring.h>
#include <coral/lang/SMD01/smd2lang.h>
%}
%init %{
 coral::init_coral_framework();
%}
#ifndef H_CORAL_CORAL
#define H_CORAL_CORAL
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.

#if defined(_WIN32) || defined(__WIN32__) || defined(__MINGW32__)
# define PLATFORM_WINDOWS
# define CORAL_HIDDEN
# ifdef COMPILING_coral
#  define EXPORT_coral __declspec(dllexport)
# else
#  define EXPORT_coral __declspec(dllimport)
# endif
# define EXPORTFWD_coral EXPORT_coral
#else
# define PLATFORM_UNIX
# ifdef HAVE_CPLUSPLUS_VISIBILITY
#  define EXPORT_coral __attribute__((visibility("default")))
#  define CORAL_HIDDEN  __attribute__((visibility("hidden")))
#  define EXPORTFWD_coral
# else
#  define EXPORT_coral
#  define CORAL_HIDDEN
#  define EXPORTFWD_coral
# endif
#ifdef linux
# define CFG_HAS_BACKTRACE
#endif
#endif


namespace coral {

	extern std::locale clocale;
/*
 * Coral general C++ defines, declarations and so on
 */

#define ARRAY_SIZE(arr) (sizeof(arr)/sizeof(arr[0]))

/* Repeatedly calls "x" until it does not throw anything. */
#define MUST_WORK(x) do { try { (x); } catch (...) { continue; } break; } while (1)

/* Can throw anything, we ignore it. Must still be executed.*/
#define MAY_FAIL(x) do { try { (x); } catch (...) { ; } } while (0)

/* Can throw anything, we ignore it. Not executed in non-debug mode. */
#ifndef NDEBUG
#define DEBUG_MAY_FAIL(x) do { try { (x); } catch (...) { ; } } while (0)
#else
#define DEBUG_MAY_FAIL(x) do { ; } while(0)
#endif

#define NAMESPACE_BEGIN(x) namespace x {
#define NAMESPACE_END() }

#define DELEGATE_FUNC(type, function, constness, variable) \
inline type function() constness { return this->variable.function(); }
#define DELEGATE_TYPE(variable, variable_name, name) \
typedef variable::variable_name name
#define DELEGATE_CONST_TYPE(variable, variable_name, name) \
typedef const variable::variable_name name

#ifdef __cplusplus
#define BEGINCPP extern "C" {
#define ENDCPP   }
#else
#define BEGINCPP 
#define ENDCPP   
#endif

#ifndef SWIG

/* Initializes the Coral framework. Returns true on success. */
int EXPORT_coral init_coral_framework() throw ();

/** Debug function to print backtrace. Required by coral_assert. */
void EXPORT_coral print_backtrace() throw ();

#endif

}

#endif
#ifndef H_CORAL_REF
#define H_CORAL_REF
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.


namespace coral {

     class EXPORT_coral ref_counted {
     private:
	  /** Reference count for garbage collection.
	      See REF_COUNTING.TXT for a description. */
	  mutable std::size_t refcount;
     public:
	  ref_counted() : refcount(1) { ; }
	  virtual ~ref_counted() throw () {
	    ;
	  }
	  void ref() const throw () { ++this->refcount; }
	  void unref() const throw ();
	  int get_ref_count() const throw () { return this->refcount; }
	  long get_id() const throw () { return reinterpret_cast<long>(this); }
	  /* Strange but needed, because of downcast/upcast issues in SWIG */
	  bool __eq__(const ref_counted* p) const throw () {
	       return this == p;
	  }
	  bool __ne__(const ref_counted* p) const throw () {
	       return this != p;
	  }
	  int __hash__() const throw () {
	       return reinterpret_cast<long>(this);
	  }
     };

#ifndef SWIG

	/* Keeps a ref() on an element, and drops it at destructor. */ 
	class EXPORT_coral temp_ref : public noncopy {
	private:
		const ref_counted* element;
		temp_ref(const temp_ref&);
		temp_ref& operator=(const temp_ref&);
	public:
		temp_ref(const ref_counted* e) : element(e) { if (e) e->ref(); }
		~temp_ref() throw () { if (this->element) this->element->unref(); }
		void release() throw () { this->element = NULL; }
	};

	/*
	 * Drops a ref upon destruction, unless told to release (forget) the
	 * pointer.
	 */
     template<class T>
	class EXPORT_coral autounref {
		T* ptr;
	public:
		autounref() throw () { ptr = NULL; }
		explicit autounref(T* e) throw () : ptr(e) { ; }
		T* release() throw () { T* t = ptr; ptr = NULL; return t; }
		T* get() const throw () { return ptr; }
		T* operator->() const throw () { return ptr; }
		autounref<T>& operator=(const autounref<T>& e) throw () {
		     if (&e == this) return *this;
		     if (ptr) {
			  if (ptr == e.ptr) return *this;
			  ptr->unref();
		     }
		     ptr = e.ptr;
		     const_cast<autounref<T>&>(e).ptr = NULL;
		     return *this;
		}
		autounref<T>& operator=(const T* e) throw () {
			if (e == ptr) return *this;
			if (ptr) {
			     if (ptr == e) return *this;
			     ptr->unref();
			}
			ptr = const_cast<T*>(e);
			return *this;
		}

		bool operator!() const throw () {
		     return ptr == NULL;
		}

		~autounref() throw () {
			if (ptr)
				ptr->unref();
		}
		
		autounref(const autounref<T>& a) throw () {
			if (&a != this) {
				ptr = a.ptr;
				const_cast<autounref<T>&>(a).ptr = NULL;
			}
		}
	template <class T2>
	     autounref<T>& operator=(const autounref<T2>& e) throw () {
	     if (ptr) {
		  if (ptr == e.get()) return *this;
		  ptr->unref();
	     }
	     ptr = e.get();
	     const_cast<autounref<T2>&>(e).release();
	     return *this;
	}
	};


#endif

}

#endif
#ifndef H_CORAL_LIST
#define H_CORAL_LIST
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.

#ifndef SWIG


struct ListPtr;

struct EXPORT_coral List_Base {
private:
	friend struct ListPtr;
	friend struct List;
	struct List_Base* next;
	struct List_Base* prev;
public:
	List_Base() throw () : next(this), prev(this) {
		;
	}
	List_Base* getNext() throw () { return this->next; }
	List_Base* getPrev() throw () { return this->prev; }
	const List_Base* getNext() const throw () { return this->next; }
	const List_Base* getPrev() const throw () { return this->prev; }
	List_Base(struct List_Base* add_to_this) throw () {
		struct List_Base* n;
		n = add_to_this->next;
		this->next = n;
		this->prev = add_to_this;
		n->prev = this;
		add_to_this->next = this;
	}
	~List_Base() throw () { this->drop(); }
	void drop() throw () {
		struct List_Base* n = this->next;
		struct List_Base* p = this->prev;
		n->prev = p;
		p->next = n;
		this->next = this;
		this->prev = this;
	}
	bool isEmpty() const throw () {
		return this == this->next;
	}
	/** Add elem (which must be detached) to this. */
	void add(struct List_Base* elem) throw () {
		coral_assert(elem->isEmpty());
		struct List_Base* n = this->next;
		this->next = elem;
		elem->prev = this;
		n->prev = elem;
		elem->next = n;
	}
	/** Add behind */
	void add_back(struct List_Base* elem) throw () {
		coral_assert(elem->isEmpty());
		struct List_Base* p = this->prev;
		this->prev = elem;
		elem->next = this;
		p->next = elem;
		elem->prev = p;
	}
	/** Move elem (which can be on another list) to this. */
	void move(struct List_Base* elem) throw () {
		elem->drop();
		this->add(elem);
	}

	/* NOTE this is a linear operation! */
	int size() const {
		int thesize = 0;
		const struct List_Base* h = this->next;
		while (h != this) {
			++thesize;
			h = h->next;
		}
		return thesize;
	}


};


struct EXPORT_coral ListPtr {
private:
	struct List_Base* cur;
public:
	ListPtr(List_Base* l) : cur(l) { ; }
	~ListPtr() throw () { ; }

	bool operator==(const ListPtr& e) const {
		return this->cur == e.cur;
	}
	bool operator!=(const ListPtr& e) const {
		return this->cur != e.cur;
	}
	struct List_Base* operator*() throw () { return this->cur; }

#ifndef SWIG
	struct ListPtr& operator++() throw () { this->cur = this->cur->next; return *this; }
	struct ListPtr operator++(int) throw () { ListPtr t = *this; this->cur = this->cur->next; return t; }
	struct ListPtr& operator--() { this->cur = this->cur->prev; return *this; }
	struct ListPtr operator--(int) throw () { ListPtr t = *this; this->cur = this->cur->prev; return t; }

#endif


};


struct EXPORT_coral List : public List_Base {
public:
	List() : List_Base() { ; }
	List(List* l) : List_Base(l) { ; }
	typedef struct ListPtr iterator;
	struct ListPtr begin() throw () { return ListPtr(this->next); }
	struct ListPtr end() throw () { return ListPtr(this); }

};

#endif

#endif

#ifndef H_CORAL_PYTHON_COMPAT
#define H_CORAL_PYTHON_COMPAT
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.



/* Supports Python iterator protocol */
template <class Iterator, class T> class EXPORT_coral PythonSequenceIteratorDirect : public coral::ref_counted {
 private:
	ref_counted* ref;
	Iterator i;
	Iterator end;
 public:
	PythonSequenceIteratorDirect(ref_counted* r, Iterator begin, Iterator theend) throw () : ref_counted(), ref(r), i(begin), end(theend) {
		coral_assert(r);
		r->ref();
	}
	virtual ~PythonSequenceIteratorDirect() throw () {
	     this->ref->unref();
	}
	PythonSequenceIteratorDirect* __iter__() throw () { return this; }
	T next() throw () {
		if (this->i == this->end)
			return NULL;
		T f = (*this->i);
		++this->i;
		return f;
	}
};


/* Supports Python iterator protocol */
/* This has the difference in that it uses .second in next() */
template <class Iterator, class T> class EXPORT_coral PythonSequenceIteratorSecond : public coral::ref_counted {
 private:
	ref_counted* ref;
	Iterator i;
	Iterator end;
 public:
	PythonSequenceIteratorSecond(ref_counted* r, Iterator begin, Iterator theend) throw () : ref_counted(), ref(r), i(begin), end(theend) {
		coral_assert(r);
		r->ref();
	}
	virtual ~PythonSequenceIteratorSecond() throw () {
	     this->ref->unref();
	}
	PythonSequenceIteratorSecond* __iter__() throw () { return this; }
	T next() throw () {
		if (this->i == this->end)
			return NULL;
		T f = (*this->i).second;
		++this->i;
		return f;
	}
};

#endif
/*
 * autounref
 */
%define UNREF_WRAP(classname)
%typemap(python,out) coral::autounref<coral:: ## classname> {
     $result = SWIG_NewPointerObj((void *) result.get(), SWIGTYPE_p_coral__ ## classname, 0);
};
%enddef

UNREF_WRAP(Project)
UNREF_WRAP(Language)
UNREF_WRAP(Package)
UNREF_WRAP(Collection)
UNREF_WRAP(OrderedSet)
UNREF_WRAP(Set)
UNREF_WRAP(Sequence)
UNREF_WRAP(Bag)
UNREF_WRAP(Element)
UNREF_WRAP(Atom)
UNREF_WRAP(ElementDefinition)
UNREF_WRAP(EnumerationDefinition)
UNREF_WRAP(DatatypeDefinition)
UNREF_WRAP(AbstractElementDefinition)

/*
 * Coral UUID
 */

%typemap(python,out) const uuid_t& {
	$result = PyString_FromString($1->c_str());
}

/*
 * std::string
 */
%typemap(python,out) std::string {
	$result = PyString_FromString($1.c_str());
}

%typemap(python,out) const std::string& {
	$result = PyString_FromString($1->c_str());
}

// new swig version 1.3.2x requires fancy typecheck junk
// the objects we receive can in fact be whatever
// but we are guaranteed ( by the %typechecks below)
// that at least it should be possible to convert it
%typemap(python,in) std::string& (std::string temp) {
     PyObject* p1 = PyObject_Str($input);
     temp = std::string(PyString_AsString(p1));
     $1 = &temp;
     Py_DECREF(p1);
}

%typemap(python,in) std::string (std::string temp) {
     PyObject* p1 = PyObject_Str($input);
     temp = std::string(PyString_AsString(p1));
     $1 = &temp;
     Py_DECREF(p1);
}

%typemap(python,in) int {
	$1 = PyInt_AsLong($input);
}

// We don't use floats in Coral, so only converting Double is ok...
%typemap(python,in) double {
	$1 = PyFloat_AsDouble($input);
}

%typemap(python,in) bool {
	$1 = PyObject_IsTrue($input);
}



// new swig version 1.3.2x requires fancy typecheck junk
%typecheck(SWIG_TYPECHECK_CHAR) std::string {
     $1 = ($input->ob_type->tp_str != NULL);
}

%typemap(typecheck) std::string& = std::string;
%typemap(typecheck) const std::string = std::string;
%typemap(typecheck) const std::string& = std::string;

%typecheck(SWIG_TYPECHECK_CHAR) int {
  PyNumberMethods* nb;
  $1 = ((nb = $input->ob_type->tp_as_number) != NULL && nb->nb_int != NULL);
}

// We don't use floats in Coral, so only checking double is OK.
%typecheck(SWIG_TYPECHECK_CHAR) double {
  PyNumberMethods* nb;
  $1 = ((nb = $input->ob_type->tp_as_number) != NULL && nb->nb_float != NULL);
}

// accept anything as bool
%typecheck(SWIG_TYPECHECK_CHAR) bool {
  $1 = 1;
}

%typecheck(SWIG_TYPECHECK_CHAR) coral::modelcopy_map_t* {
  $1 = PyDict_Check($input);
}

/*
 * unicodestring
 */
%typemap(python,out) coral::unicodestring {
	$result = $1.toPython();
}

%typemap(python,out) const coral::unicodestring& {
	$result =  $1->toPython();
}

%typemap(python,in) coral::unicodestring& (coral::unicodestring temp) {
     temp = coral::unicodestring($input);
     $1 = &temp;
}

/*
 * std::vector<std::string>
 */
%typemap(python,out) std::vector<std::string> {
	$result = PyList_New($1.size());
	if (!$result) {
		$result = Py_None;
		Py_INCREF($result);
	} else {
		int pos = 0;
		for (std::vector<std::string>::const_iterator i = $1.begin();
		     i != $1.end();
		     ++i, ++pos) {
			PyObject* s = PyString_FromString((*i).c_str());
			if (!s) {
				Py_DECREF($result);
				$result = Py_None;
				Py_INCREF($result);
				break;
			}
			(void) PyList_SetItem($result, pos, s);
		}
	}
}

%typemap(python,in) std::vector<std::string> {
	if (!PyList_Check($input)) {
		PyErr_SetString(PyExc_TypeError,
				"Must receive list as input!");
		return NULL;

	}
	int size =  PyList_Size($input);
	for (int i = 0; i < size; ++i) {
		PyObject* o = PyList_GetItem($input, i);
		if (!PyString_Check(o)) {
			PyErr_SetString(PyExc_TypeError,
					"Must receive list of strings as input!");
			return NULL;
		}
		$1.push_back(PyString_AsString(o));
	}
}

/*
 * std::map<Element*, Element*>
 */
%typemap(python,out) coral::modelcopy_map_t* {
	$result = PyDict_New();
	if (!$result) {
		$result = Py_None;
		Py_INCREF($result);
	} else {
	}
}

%typemap(python, in) coral::modelcopy_map_t* {
     if (!PyDict_Check($input)) {
	  PyErr_SetString(PyExc_TypeError,
			  "Must receive dict as input!");
	  return NULL;
     } else {
	  $1 = new coral::modelcopy_map_t();
     }
}

%typemap(python, freearg) coral::modelcopy_map_t* {
     if ($1) {
	  delete $1;
     }
}

%typemap(python, argout) coral::modelcopy_map_t* {
     if ($1) {
	  for (std::map<const coral::Element*, const coral::Element*>::const_iterator i = $1->begin();
	       i != $1->end();
	       ++i) {
	       const coral::Element* e1 = i->first;
	       const coral::Element* e2 = i->second;
	       // BUG should unref in case of exceptions
	       PyObject* p1 =
		    SWIG_NewPointerObj(const_cast<coral::Element*>(e1),
				       SWIGTYPE_p_coral__Element,
				       0);
	       PyObject* p2 =
		    SWIG_NewPointerObj(const_cast<coral::Element*>(e2),
				       SWIGTYPE_p_coral__Element,
				       0);
	       int err = PyObject_SetItem($input, p1, p2);
	       coral_assert(err == 0);
	  }
     }
}



/*
 * std::pair<double, double>
 */
%typemap(python,out) std::pair<double, double> {
	$result = PyList_New(2);
	if (!$result) {
		$result = Py_None;
		Py_INCREF($result);
	} else {
		PyObject* d1 = PyFloat_FromDouble($1.first);
		PyObject* d2 = PyFloat_FromDouble($1.second);
		if (d1 && d2) {
			(void) PyList_SetItem($result, 0, d1);
			(void) PyList_SetItem($result, 1, d2);
		} else {
			if (d1) Py_DECREF(d1);
			if (d2) Py_DECREF(d2);
			Py_DECREF($result);
			$result = Py_None;
			Py_INCREF($result);
		}
	}
}


/*
 * std::vector<coral::Project*>
 */
%typemap(python,out) std::vector<coral::Project*> {
	$result = PyList_New($1.size());
	if (!$result) {
		$result = Py_None;
		Py_INCREF($result);
	} else {
		int pos = 0;
		for (std::vector<coral::Project*>::const_iterator i = $1.begin();
		     i != $1.end(); ++i, ++pos) {
			PyObject* s =
				SWIG_NewPointerObj(*i,
						   SWIGTYPE_p_coral__Project,
						   0);
			if (!s) {
				Py_DECREF($result);
				$result = Py_None;
				Py_INCREF($result);
				break;
			}
			(void) PyList_SetItem($result, pos, s);
		}
	}
}

/*
 * FILE*
 */
%typemap(python,in) std::FILE* {
     $1 = PyFile_AsFile($input);
}

%typemap(python,out) std::size_t {
     $result = PyInt_FromLong($1);
}

/* This doesn't work for some reason... */
%typemap(python,out) const subclasses_type& {
  $result = PyList_New($input.size());
  coral_assert($result);
  for (int pos = 0; $input::const_iterator i = $input.begin(); i != $input.end(); ++i, ++pos) {
    PyObject* obj;
    obj = SWIG_NewPointerObj((void *) *i, SWIGTYPE_p_Coral__AbstractElementDefinition, 0);
    coral_assert(obj);
    PyList_SetItem($result, pos, obj);
  }
  return $result;
}

%template(ModelFeatureIterator) PythonSequenceIteratorSecond<coral::Element::slots_type::iterator, coral::Slot*>;

%template(PropertyIterator) PythonSequenceIteratorDirect<coral::ElementDefinition::ordered_properties_type::const_iterator, coral::Property*>;


%template(SuperClassesIterator) PythonSequenceIteratorDirect<coral::AbstractElementDefinition::superclasses_type::iterator, coral::AbstractElementDefinition*>;
%template(SubClassesIterator) PythonSequenceIteratorDirect<coral::AbstractElementDefinition::subclasses_type::iterator, coral::AbstractElementDefinition*>;
%template(AllSuperClassesIterator) PythonSequenceIteratorDirect<coral::AbstractElementDefinition::topologic_all_superclasses_type::iterator, coral::AbstractElementDefinition*>;

%template(MDAEDIterator) PythonSequenceIteratorDirect<coral::Package::elements_type::iterator, coral::AbstractElementDefinition*>;

%template(OperationsIterator) PythonSequenceIteratorDirect<coral::AbstractElementDefinition::operations_type::iterator, coral::PyOperation*>;


%exception __getitem__ {
     try {
	  $action
     } catch (std::out_of_range &e) {
	  PyErr_SetString(PyExc_IndexError, e.what());
	  return NULL;
     }
}

%exception __getSlotByNameInsert__ {
     try {
	  $action
     } catch (std::invalid_argument &e) {
	  PyErr_SetString(PyExc_IndexError, e.what());
	  return NULL;
     }
}

%exception __getSlotByNameInsertElement__ {
     try {
	  $action
     } catch (std::invalid_argument &e) {
	  PyErr_SetString(PyExc_IndexError, e.what());
	  return NULL;
     }
}

#ifndef H_CORAL_ELEMENT
#define H_CORAL_ELEMENT
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.











namespace coral {

	class EXPORTFWD_coral ElementPrimitive;
	class EXPORTFWD_coral Collection;
	class EXPORTFWD_coral Element;
	class EXPORTFWD_coral AtomIterator;
	class EXPORTFWD_coral Project;
	class EXPORTFWD_coral OrderedSet;

#ifndef SWIG
	typedef List all_modelelements_type;

#ifdef COMPILING_coral
	/* only the core of Coral needs to have these */

	/* Protects all_modelelements. */
	EXTERN_MUTEX(mutex_modelelements);

	extern all_modelelements_type all_modelelements;
#endif
	std::size_t EXPORT_coral all_modelelements_size();
#endif

	class EXPORTFWD_coral Slot;
	class EXPORT_coral Atom : public ref_counted, public noncopy {
	protected:
		AbstractElementDefinition* m_type;
		explicit Atom(AbstractElementDefinition* type);
	public:
		virtual ~Atom() throw ();
		coral::AbstractElementDefinition* __type__() const throw () {
			return m_type;
		}
		/*
		 * Checks if @this owns @obj.
		 * Succeeds even though there is no path back
		 * from @obj to @this. (cf. __isConnectedTo__)
		 */
		virtual bool __isPart__(const Atom* obj,
				    bool recurse = true) const throw () {
			return this == obj;
		}
		virtual coral::unicodestring __str__() const ;

		
		coral::autounref<coral::Set> __asSet__()
			throw (std::exception);
		bool oclIsKindOf(const coral::AbstractElementDefinition* e) const throw () {
			return this->__type__()->isSubclass(e);
		}

		// ALL BELOW MOVED FROM ELEMENTPRIMITIVE

	     /* Returns the integer representation. Valid for enumerations
		and integers. */
	     virtual int __int__() const throw ();
	     long __long__() const throw () { return __int__(); }
	     /* Returns the floating-point approximation. */
	     virtual double __float__() const throw ();
	     virtual int __getEValue__() const throw ();
	     virtual coral::unicodestring __getEValueByName__();
	     virtual void __setEValue__(int newValue);
	     virtual void __setEValueByName__(const char* newValue);
	     virtual PyObject* __getPyValue__() const throw ();
	     virtual void __setPyValue__(PyObject*) throw ();
	     virtual int __getIValue__() const throw ();
	     virtual void __setIValue__(int newValue);
	     virtual double __getDValue__() const throw ();
	     virtual void __setDValue__(double newValue) throw ();
	     virtual void __setString__ (const coral::unicodestring& newValue);
	     virtual const coral::unicodestring& __getString__() const throw ();
	};

#ifndef SWIG
	class slots_lessthancmp {
	public:
		bool operator()(const Property* const& x, const Property* const & y) const throw () {
			return x < y;
		}
	};
#endif

     class EXPORT_coral Element : public Atom, private List {
#ifdef COMPILING_coral
		friend void x__garbage_collect();
#endif
     public:
	     typedef auto2map<const Property*,
			      Slot*,
			      slots_lessthancmp > slots_type;
     protected:
	     mutable DCE_UUID m_uuid;
     private:
	     Slot& createSlot(Property*) throw (coral::bad_slot,
						std::exception);
	     slots_type slots;
     public:
	  explicit Element(coral::ElementDefinition* type, bool generate_element_attributes = true);
	  virtual ~Element() throw ();

	  // Wrap m_uuid instead of superclassing DCE_UUID. This makes python
	  // faster...
	  const coral::uuid_t& __XMIid__() const { return m_uuid.__XMIid__(); }
	  void __resetXMIid__() throw () { m_uuid.__resetXMIid__(); }
#ifndef SWIG
	  void swapXMIid(std::string& newID) throw () { m_uuid.swapXMIid(newID); }
#endif
	  /*
	   * Amazing stuff. Will switch types of a Element.
	   * Caller is responsible for any dataloss, though.
	   * It will keep the slots which are common, and of the same
	   * type. The rest will be thrown away, and new slots
	   * will be created (lazily) with default, empty values.
	   * Returns true if switch was successful.
	   */
	  bool __switchType__(coral::ElementDefinition* type);

	  bool isWellFormed() /* const */;
	  bool isWellFormedRecursive() /* const */;

	  /*
	   * Returns element which is this element's parent,
	   * i.e. the parent has a composition relation to this
	   * There can be only one, due to our assumption
	   * that composition forms a tree.
	   * Won't traverse up ElementPrimitives, since they don't
	   * have a pointer upwards. :(
	   * Returns NULL if there is no parent.
	   */
	  Element* __parent__() const throw ();
	  /*
	   * Returns the slot in which the parent element is,
	   * or NULL if no parent.
	   */
	  Slot* __parentSlot__() const throw ();

	  /*
	   * Returns top-most __parent__, or itself.
	   */
	  Element* __getRoot__() const throw (coral::coral_error);

	  coral::autounref<coral::Set> __getAllParts__();
	  
#ifndef SWIG
	  AtomIterator DFSIterator();
#endif

	  /*
	   * Returns Project if Element is root
	   */
	  coral::Project* __getProject__() const throw ();
	  /*
	   * Sets the Project of an element.
	   */
	  void __setProject__(coral::Project*) throw ();
	  /*
	   * Returns Project of topmost element
	   */
	  coral::Project* __getRootsProject__() const throw (coral::coral_error);

	  void __setXMIid__(const coral::uuid_t&);
	  /* Returns feature iff it exists */
	  Slot* __hasSlotByName__(const char* fname) const throw ();
	  /* Returns feature if it exists, otherwise a read-only empty
	     slot. Always succeeds with valid @fname! */
	  const Slot& __getSlotRO__(const char* fname) const
		  throw (coral::bad_slot);
	  /* Creates feature if possible and it doesn't exist, and
	     returns it. */
	  Slot& __getSlotByName__(const char* fname)
		  throw (coral::bad_slot, std::exception);
	  /*
	   * For const elements.
	   */
#ifndef SWIG
	  const Slot& __getSlotByName__(const char* fname) const
		  throw (coral::bad_slot, std::exception) {
		  return __getSlotRO__(fname);
	  }
#endif
	     Slot& __getSlotByOppositeName__(const coral::ElementDefinition* e,
					     const char* fname)
		     throw (coral::bad_slot, std::exception);
	  /* Faster python, and we need it for proper TM stuff
	     as well. Argh! */
	  void __getSlotByNameInsert__(const char* fname, PyObject*)
		  throw (std::invalid_argument, std::exception);
	  void __getSlotByNameInsertElement__(const char* fname, Element*)
		  throw (std::invalid_argument, std::exception);
	  /* Returns non-null if the given feature is defined, and is
	     a SlotOneElement */
	  Atom* __getSlotOneElement__(const char* fname) const throw ();

	  coral::ElementDefinition* __type__() const throw () {
	       return static_cast<coral::ElementDefinition*>(this->Atom::__type__());
	  }
	  bool __isPart__(const Atom*, bool recurse = true) const throw ();
#ifndef SWIG
	  bool __isPart__(const Element* obj, bool recurse = true) const throw ();
#endif
	  /*
	   * Returns true if @this owns @obj, or @obj owns @this.
	   */
	  bool __isConnectedTo__(const Element* obj) const throw ();
#ifndef SWIG

	  DELEGATE_FUNC(slots_type::iterator, begin,, slots);
	  DELEGATE_FUNC(slots_type::iterator, end,, slots);
	  DELEGATE_TYPE(slots_type, iterator, iterator);

	  DELEGATE_FUNC(slots_type::const_iterator, begin, const, slots);
	  DELEGATE_FUNC(slots_type::const_iterator, end, const, slots);
#endif


	  /** Resets all slots to their default.
	      Note that this might imply you need to ref() to keep
	      the element alive. */
	  void __resetAttrs__();
#ifndef SWIG
	  void __detachUsFromEverybodyElse__(const std::set<Element*>&
					     all_elems_participating);
	  virtual coral::unicodestring __str__() const;
#endif

	  PythonSequenceIteratorSecond<slots_type::iterator, coral::Slot*>* __all_slots__() {
	       return new PythonSequenceIteratorSecond<slots_type::iterator, coral::Slot*>(this, this->slots.begin(), this->slots.end());
	  }
	  PythonSequenceIteratorDirect<coral::ElementDefinition::ordered_properties_type::const_iterator, coral::Property*>* __properties__() {
	       return this->__type__()->__properties__();
	  }
       
     };





     class EXPORT_coral Slot : public noncopy {
	private:
	     void insert_py(PyObject* e, int index = -1, int other_index = -1,
			    bool unidirectional = false) throw (std::exception);
     protected:
	  Property* mmproperty;
	  /* This is NULL for RO-slots */
	  Element* element;
	   bool TRANSACTION() throw ();
     public:
	  coral::Element* getElement() const throw () { return this->element; }
	  Slot(coral::Element* e, coral::Property* f);
	  virtual ~Slot() throw ();
	  virtual std::auto_ptr<Slot> clone() = 0;
	  virtual coral::Collection* getCollection() const throw ();
	  coral::Property* getProperty() const throw () {
	       return this->mmproperty;
	  }

	  /*
	   * Swaps slot contents with another slot.
	   * ONLY used by the TransactionManager
	   * Note that this can't be put around
	   * #ifdef COMPILING_coral because it's virtual, I think.
	   */
#ifndef SWIG
	   virtual void swapSlotContents(Slot*) throw () = 0;
#endif

	     /**
	      * Returns an iterator object suitable for Python.
	      * Don't call directly.
	      */
	  coral::Iterator* __iter__();
	  /** Returns a new string object giving a string representation
	      of the current value. */
	  /** Returns element at given index. */
	  virtual Atom* __getitem__(int index) const throw (std::out_of_range);
	  virtual coral::unicodestring __str__() const = 0;
#ifndef SWIG
	  Atom* operator[](int index) const throw (std::out_of_range) {
		  return this->__getitem__(index);
	  }
#endif
	  virtual void __setitem__(int index, Atom*) throw (std::out_of_range, std::exception);
	  /* Returns amount of elements. */
	  virtual std::size_t __len__() const throw ();
	  /* OCL compat. */
	  std::size_t size() const throw () { return this->__len__(); }
	  /* Resets the feature. */
	  virtual void resetSlot() = 0;

	  /*
	    Convenience, so user doesn't have to cast to correct type...
	    Note that they are all virtual functions, which default
	    to coral_assert(0). Override in subclasses _only_ where necessary.
	  */

	  /** Adds an element e at a specified index.
	      If index is 0 or positive, e will be inserted at
	      that location. If index is negative, it will be inserted
	      counting from the back. Default is append.
	  */
	  virtual void insert(Atom* e, int index = -1, int other_index = -1, bool unidirectional = false) throw (std::exception);

	  /** Removes the first occurence of e, returning its (old) index.
	      NOTE! e might be unavailable after this! */
	  virtual int drop(Atom* e, bool unidirectional = false) throw (std::exception);
	  int remove(Atom* e, bool unidirectional = false) throw (std::exception) { return this->drop(e, unidirectional); }
	  /** Removes the element at specifed index.
	      If index is 0 or positive, element will be removed from
	      that location. If index is negative, element will be removed
	      counting from the back.
	      See also remove_keepref().
	  */
	  void drop_index(int index, bool unidirectional = false) {
	       this->remove_keepref(index, unidirectional)->unref();
	  }


	  /** Add, for unordered connections. */
	  void add(Atom* e, bool unidirectional = false) throw (std::exception) {
	       //coral_assert(this->getProperty()->isOrdered());
	       coral_assert(!this->getProperty()->isAtMostOne());
	       this->insert(e, -1, -1, unidirectional);
	  }
	  /** Add, for ordered connections. */
	  void append(Atom* e, bool unidirectional = false) throw (std::exception) {
	       //coral_assert(!this->getProperty()->isOrdered());
	       coral_assert(!this->getProperty()->isAtMostOne());
	       this->insert(e, -1, -1, unidirectional);
	  }

	  // CRL-451 compatibility with plain Python objects
	  void add(PyObject* e, bool unidirectional = false)
	       throw (std::exception) {
	       coral_assert(!this->getProperty()->isAtMostOne());
	       this->insert_py(e, -1, -1, unidirectional);
	  }
	  void append(PyObject* e, bool unidirectional = false)
	       throw (std::exception) {
	       coral_assert(!this->getProperty()->isAtMostOne());
	       this->insert_py(e, -1, -1, unidirectional);
	  }
#ifndef SWIG
	  void helper_insert(ElementPrimitive* old,
			     PyObject* e, int index = -1, int other_index = -1,
			     bool unidirectional = false)
	       throw (std::exception);

	  /** Remove, when you want to use the returned element.
	      You must manually do unref() on the return value. */
	  virtual Atom* remove_keepref(int index, bool unidirectional = false);
#endif
     };


#ifndef SWIG
/* A zillion different Slots... */


     class EXPORT_coral SlotManyElements: public Slot {
     protected:
	  friend class Element;
	  friend void unmark(__gnu_cxx::hash_set<Atom*, ptr_hash<Atom> >& element_blacks, Element* white);

	  Collection* collection;
	  void insert(Atom* e, int index, int other_index, bool unidirectional = false) throw (std::exception);
	  int drop(Atom* e, bool unidirectional = false) throw ();
	  Atom* __getitem__(int index) const throw (std::out_of_range);
	  void __setitem__(int index, Atom*) throw (std::out_of_range, std::exception);
	  Atom* __remove_continued(Atom* child, bool unidirectional) throw ();
	  Atom* remove_keepref(int index, bool unidirectional = false) throw ();

     public:
	  SlotManyElements(Element* e, Property* f, coral::Collection* m);
	  Collection* getCollection() const throw ();
	  virtual ~SlotManyElements() throw ();

	   virtual void swapSlotContents(Slot*) throw ();

	  std::size_t __len__() const throw ();
	  virtual coral::unicodestring __str__() const;
	  void resetSlot();
	  std::auto_ptr<coral::Slot> clone();
     };

     class EXPORT_coral SlotOneElement: public Slot {
     private:
	  Atom* value;
     public:
	  SlotOneElement(Element* e, Property* f);
	  virtual ~SlotOneElement() throw ();
	  std::size_t __len__() const throw ();
	  virtual coral::unicodestring __str__() const;
	  virtual void resetSlot();

	   virtual void swapSlotContents(Slot*) throw ();

	  Atom* __getitem__(int index) const throw (std::out_of_range);

	  void insert(Atom* e, int index = -1, int other_index = -1, bool unidirectional = false) throw (std::exception);
	  int drop(Atom* e, bool unidirectional = false) throw ();
	  Atom* remove_keepref(int index, bool unidirectional = false);
	  std::auto_ptr<coral::Slot> clone();
     };


#endif

     /*
      * Elements of primitive type
      */

    class EXPORT_coral ElementPrimitive : public Atom {
	protected:
	      void TRANSACTION() throw ();
	     ElementPrimitive(DatatypeDefinition*);
     public:
	     virtual ~ElementPrimitive() throw ();
	     coral::DatatypeDefinition* __type__() const throw () {
		     return static_cast<coral::DatatypeDefinition*>(this->Atom::__type__());
	     }
	     /** Makes a new Atom with the same values. */
	     virtual coral::ElementPrimitive* clone() const = 0;
	     /*
	      * Resets a primitive to its default value
	      */
	     virtual void reset() = 0;
	     /** Swaps values with atom, which must be of
	      * the exact same subtype as this.
	      */
	     virtual void swapWithAtom(ElementPrimitive*) throw () = 0;
	     virtual bool operator==(const ElementPrimitive&) const throw () = 0;
	     bool operator!=(const ElementPrimitive& o) const throw () {
	       return !(*this == o);
	     }
	     bool operator==(const Element&) const throw () { return false; }
	     bool operator!=(const Element&) const throw () { return true; }

	     bool operator==(const PyObject*) const throw ();
	     bool operator!=(const PyObject* p) const throw () {
		  return !(*this == p);
	     }
	     virtual bool operator<(const ElementPrimitive&) const throw () = 0;
	     virtual bool operator<=(const ElementPrimitive& o) const throw () = 0;
	     bool operator>(const ElementPrimitive& o) const throw () {
		  return o <= *this;
	     }
	     bool operator>=(const ElementPrimitive& o) const throw () {
		  return o < *this;
	     }
	     virtual std::size_t __len__() const throw () = 0;
#ifndef SWIG
	     virtual coral::unicodestring __str__() const = 0;
#endif
	     coral::unicodestring __unicode__() const { return __str__(); }
     };

#ifndef SWIG

     class EXPORT_coral ElementPyObject: public ElementPrimitive {
     private:
	     PyObject* obj;
     public:
	     ElementPyObject(DatatypeDefinition*);
	     virtual ~ElementPyObject() throw ();

	     virtual void reset();
	     virtual PyObject* __getPyValue__() const throw ();
	     virtual void __setPyValue__(PyObject* newObj) throw ();
	     
	     virtual coral::unicodestring __str__() const;
	     virtual std::size_t __len__() const throw ();
	     virtual ElementPyObject* clone() const;
	     virtual void swapWithAtom(ElementPrimitive*) throw ();
	     virtual bool operator==(const ElementPrimitive&) const throw ();
	     virtual bool operator<(const ElementPrimitive&) const throw ();
	     virtual bool operator<=(const ElementPrimitive&) const throw ();
     };

     class EXPORT_coral ElementEnumeration: public ElementPrimitive {
     private:
	     int enumValue;
     public:
	     coral::EnumerationDefinition* __type__() const throw () {
		     return static_cast<coral::EnumerationDefinition*>(this->Atom::__type__());
	     }

	     ElementEnumeration(EnumerationDefinition* e);
	     virtual ~ElementEnumeration() throw ();

	     virtual void reset();
	     virtual int __getEValue__() const throw () {
		     return this->enumValue;
	     }
	     virtual coral::unicodestring __getEValueByName__();
	     virtual void __setEValue__(int newValue);
	     virtual void __setEValueByName__(const char* newValue);
	     
	     virtual coral::unicodestring __str__() const;
	     virtual int __int__() const throw ();
	     virtual ElementEnumeration* clone() const;
	     virtual void swapWithAtom(ElementPrimitive*) throw ();
	     virtual bool operator==(const ElementPrimitive&) const throw ();
	     virtual bool operator<(const ElementPrimitive&) const throw ();
	     virtual bool operator<=(const ElementPrimitive&) const throw ();
	     virtual std::size_t __len__() const throw ();
     };

     /*
      * BUG We should use this much much more...
      * We probably want to go to long longs anyway
      */
     typedef int coral_integer;
     typedef double coral_double;

     class EXPORT_coral ElementInteger: public ElementPrimitive {
     private:
	  coral_integer ivalue;
     public:
	  ElementInteger(DatatypeDefinition*);
	  virtual ~ElementInteger() throw ();

	  virtual void reset();
	  virtual int __int__() const throw ();
	  virtual double __double__() const throw ();

	  virtual int __getIValue__() const throw () {
	       return this->ivalue;
	  }
	  virtual void __setIValue__(int newValue) throw ();
	  virtual std::size_t __len__() const throw ();

	  virtual coral::unicodestring __str__() const;
	  virtual ElementInteger* clone() const;
	  virtual void swapWithAtom(ElementPrimitive*) throw ();
	  virtual bool operator==(const ElementPrimitive&) const throw ();
	  virtual bool operator<(const ElementPrimitive&) const throw ();
	  virtual bool operator<=(const ElementPrimitive&) const throw ();
     };

     class EXPORT_coral ElementDouble: public ElementPrimitive {
     private:
	  double dvalue;
     public:
	  ElementDouble(DatatypeDefinition*);
	  virtual ~ElementDouble() throw ();

	  virtual void reset();
	  virtual double __float__() const throw ();
	  virtual int __int__() const throw ();
	  virtual double __getDValue__() const throw () {
	       return this->dvalue;
	  }
	  virtual void __setDValue__(double newValue) throw ();
	  virtual std::size_t __len__() const throw ();

	  virtual coral::unicodestring __str__() const;
	  virtual ElementDouble* clone() const;
	  virtual void swapWithAtom(ElementPrimitive*) throw ();
	  virtual bool operator==(const ElementPrimitive&) const throw ();
	  virtual bool operator<(const ElementPrimitive&) const throw ();
	  virtual bool operator<=(const ElementPrimitive&) const throw ();
     };

     class EXPORT_coral ElementString: public ElementPrimitive {
     private:
	  unicodestring svalue;
     public:
	  ElementString(DatatypeDefinition*);
	  virtual ~ElementString() throw ();

	  virtual void reset();
	  virtual void __setString__(const unicodestring& newValue);
	  virtual const coral::unicodestring& __getString__() const throw ();

	  virtual unicodestring __str__() const;
	  virtual std::size_t __len__() const throw ();
	  virtual ElementString* clone() const;
	  virtual void swapWithAtom(ElementPrimitive*) throw ();
	  virtual bool operator==(const ElementPrimitive&) const throw ();
	  virtual bool operator<(const ElementPrimitive&) const throw ();
	  virtual bool operator<=(const ElementPrimitive&) const throw ();
     };

#endif

}

#endif
#ifndef H_CORAL_ELEMENTDEFINITION
#define H_CORAL_ELEMENTDEFINITION
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.





namespace coral {

     class EXPORTFWD_coral EnumerationDefinition;
     class EXPORTFWD_coral Property;
     class EXPORTFWD_coral AbstractElementDefinition;
     class EXPORTFWD_coral ElementDefinition;
     class EXPORTFWD_coral Atom;
     class EXPORTFWD_coral DCE_UUID;
     class EXPORTFWD_coral Element;
     class EXPORTFWD_coral Slot;
     class EXPORTFWD_coral Language;
     class EXPORTFWD_coral Package;
     class EXPORTFWD_coral Meta2ModelBridge;

      void setCorrespondingElement(coral::Meta2ModelBridge*,
					       coral::Element*);

     // This is the __meta__ hack
     class EXPORT_coral Meta2ModelBridge : public ref_counted {
	  coral::autounref<coral::Element> m_correspondingElement;
	  friend void setCorrespondingElement(coral::Meta2ModelBridge* mm,
					      coral::Element* e);
     public:
	  coral::autounref<coral::Element> getCorrespondingElement();
     };

/** The general metamodel element in Coral. */
class EXPORT_coral AbstractElementDefinition:
     public Meta2ModelBridge, public DCE_UUID {
 private:
	PyObject* bindOperation(const std::string& name);
	PyObject* bindOperation_single(const std::string& name);
	std::string m_name;
 protected:
	Package* package;
 public:
	typedef std::list<AbstractElementDefinition*> subclasses_type;
	typedef std::list<AbstractElementDefinition*> superclasses_type;
	typedef std::list<AbstractElementDefinition*> topologic_all_superclasses_type;
	typedef autounreflist<PyOperation*> operations_type;
	coral::PyOperation* getOperation(const std::string& name);
 protected:
	bool isAbstractED;
	operations_type operations;
	subclasses_type subclasses;
	superclasses_type superclasses;
	topologic_all_superclasses_type all_superclasses;
	 void recalculateAllSuperclasses();
	 void
	  recalculateForMyselfAndSubclasses(std::set<AbstractElementDefinition*>& visited);
	AbstractElementDefinition(Package*, ref_counted* type, const char* name);
     public:
	static coral::autounref<coral::AbstractElementDefinition>
	  getAbstractElementDefinitionFromSMD(const Element* e);
	void __addOperation__(coral::PyOperation*);
	void __removeOperation__(PyObject*);
	PyObject* __bindOperation__(const std::string& name,
				PyObject* self);

	bool isAbstract() { return isAbstractED; }
	void setAbstract(bool newvalue) { isAbstractED = newvalue; }

	virtual ~AbstractElementDefinition() throw ();

	const std::string& getName() const throw () {
	  return this->m_name;
	}

	void addSubclass(AbstractElementDefinition* m) {
		m->addSuperclass(this);
	}
	void addSuperclass(AbstractElementDefinition* m);
#ifndef SWIG
	const operations_type& getOperations() const throw () { return this->operations; }
	const subclasses_type& getSubclasses() const throw () { return this->subclasses; }
	const superclasses_type& getSuperclasses() const throw () { return this->superclasses; }
	const topologic_all_superclasses_type& getAllSuperclasses() const throw () { return this->all_superclasses; }
#endif
	virtual bool isDatatypeDefinition() const throw () { return false; }
	virtual bool isEnumerationDefinition() const throw () { return false; }
	Package* __package__() const throw () { return this->package; }
	Language* __language__() const throw (); 
	/** Returns true is this is a root element definition in its language */
	bool isRootDefinition() const throw ();
	/** Returns true if this is a subclass of e*/
	bool isSubclass(const AbstractElementDefinition* e) const throw ();
	/** Returns true if this is a subclass of e*/
	bool isProperSubclass(const AbstractElementDefinition* e) const throw () {
	     return (this != e) && this->isSubclass(e);
	}
	PythonSequenceIteratorDirect<coral::AbstractElementDefinition::superclasses_type::iterator, coral::AbstractElementDefinition*>* __superclasses__() {
	     return new PythonSequenceIteratorDirect<coral::AbstractElementDefinition::superclasses_type::iterator, coral::AbstractElementDefinition*>(this, this->superclasses.begin(), this->superclasses.end());
	}

	PythonSequenceIteratorDirect<coral::AbstractElementDefinition::superclasses_type::iterator, coral::AbstractElementDefinition*>* __all_superclasses__() {
	     return new PythonSequenceIteratorDirect<coral::AbstractElementDefinition::superclasses_type::iterator, coral::AbstractElementDefinition*>(this, this->all_superclasses.begin(), this->all_superclasses.end());
	}

	PythonSequenceIteratorDirect<coral::AbstractElementDefinition::subclasses_type::iterator, coral::AbstractElementDefinition*>* __subclasses__() {
	     return new PythonSequenceIteratorDirect<coral::AbstractElementDefinition::subclasses_type::iterator, coral::AbstractElementDefinition*>(this, this->subclasses.begin(), this->subclasses.end());
	}

	PythonSequenceIteratorDirect<coral::AbstractElementDefinition::operations_type::iterator, coral::PyOperation*>* __operations__() {
	     return new PythonSequenceIteratorDirect<coral::AbstractElementDefinition::operations_type::iterator, coral::PyOperation*>(this, this->operations.begin(), this->operations.end());
	}

	/** Returns true if this is a presentation metamodel element. */
	virtual bool isPresentation() const throw ();
	/** Returns new atom instance. */
	virtual Atom* newAtom() ;
	Element* newElement();
	virtual std::string __str__() const ;
	
};

     coral::autounref<coral::Set> inheritanceTree(coral::AbstractElementDefinition* e);

/**
 * Returns true if exactly either one of the two given types
 * are presentations.
 */
 bool crossesPresentationLayer(const AbstractElementDefinition*, const AbstractElementDefinition*);

class EXPORTFWD_coral Property;

class EXPORT_coral ElementDefinition: public AbstractElementDefinition {
 public:
	typedef auto2unrefmap<const char*, Property*, less_strcmp<const char*> > properties_type;
	/*
	 * Properties in order required for MOF Datatypes.
	 * A bit crude, since it makes all lists of Properties in SMD models
	 * ordered. :(
	 */
	typedef std::vector<Property*> ordered_properties_type;
 private:
	properties_type properties;
	ordered_properties_type ordered_properties;
	bool MOFDatatype;
 public:
	 void __propagateProperties(ElementDefinition* super);
	ElementDefinition(Package*, const char* name);
	virtual ~ElementDefinition() throw ();

#ifdef COMPILING_coral
	 bool _isWellFormed(/* const */ Element*) const throw();
#endif
	/* These functions should only be used when creating
	   the metamodel element. */
	 void addProperty(Property* f);
	void addPropertyRecursive(Property* f);
#ifndef SWIG
	const properties_type* getProperties() const throw ();
	 const ordered_properties_type*
	  getPropertiesInOrder() const throw ();
#endif
	Property* getPropertyByName(const char* name) const throw ();
	Property* getPropertyByOpposite(const ElementDefinition* oppo,
					const char* opponame);
	//@deprecated
	 bool isMOFDatatype() const throw () {
	  return MOFDatatype;
	}
	//@deprecated
	 void setMOFDatatype(bool newvalue) throw () {
	  MOFDatatype = newvalue;
	}
	Element* operator()(bool generate_element_attributes = true);
	bool isPresentation() const throw ();
	PythonSequenceIteratorDirect<ElementDefinition::ordered_properties_type::const_iterator, coral::Property*>* __properties__() {
	     return new PythonSequenceIteratorDirect<ElementDefinition::ordered_properties_type::const_iterator, coral::Property*>(this, this->getPropertiesInOrder()->begin(), this->getPropertiesInOrder()->end());
	}
#ifndef SWIG
	virtual Atom* newAtom();
	virtual std::string __str__() const;
	
	 void __setCorrectPropertiesOrder(ordered_properties_type& t);
#endif
};


#ifndef SWIG
     struct kind_t {
	     /* composition is (at least now) mutually excl. with association
	      * but don't count on it... */
	     static const int KIND_ORDERED = 1;
	     static const int KIND_BAG = 2;
	     static const int KIND_BIDI = 4;
	     static const int KIND_NOSERIALIZE = 8;
	     static const int KIND_ANONYMOUS = 16;
	     static const int KIND_COMPOSITION = 32;
	     static const int KIND_ASSOCIATION = 64;
	     int kind;
	     bool isOrdered() const throw () { return kind & KIND_ORDERED; }
	     bool isBag() const throw () { return kind & KIND_BAG;}
	     bool isBidirectional() const throw () {return kind & KIND_BIDI; }
	     bool isComposition() const throw () { return kind & KIND_COMPOSITION;}
	     bool isAssociation() const throw () { return kind & KIND_ASSOCIATION;}
	     bool isUnserializable() const throw () { return kind & KIND_NOSERIALIZE; }
	     bool isAnonymous() const throw () { return kind & KIND_ANONYMOUS; }
	     kind_t(const int t = 0) : kind(t) { ; }
	     const kind_t& operator|=(const int t) { kind |= t; return *this; }
     };
#endif

class EXPORT_coral Property : public Meta2ModelBridge, public noncopy {
 private:
	  /* Name of the Property */
	  std::string key;
	  /* Default value (for primitives) */
	  std::string m_default_value;
	/* What Slots of this Property should contain. */
	AbstractElementDefinition* element_kind;
	/* Top-level owner of this property. */
	AbstractElementDefinition* owner;
	/* Opposite Property */
	Property* other;
	/* A read-only slot with no contents. Used for Element.__getSlotRO__() */
	Slot* read_only_slot;
	kind_t kind;
	int mult_lower;
	// -1 for unlimited
	int mult_upper;
	Property* __findOtherProperty(AbstractElementDefinition*, const char*) const throw ();
	friend class ElementDefinition;
	void __init(const char* _key, AbstractElementDefinition* _element_kind,
		    const char* other_key,
		    int m_lower, int m_upper,
		    kind_t k,
		    const std::string& default_value);
 public:
	virtual ~Property() throw ();
	AbstractElementDefinition* getOwner() const throw () { return owner; }
	Slot* getROSlot() const throw () {
		coral_assert(this->read_only_slot);
		return this->read_only_slot;
	}
	const std::string& getDefaultValue() const throw () { return m_default_value; }
	bool isMultSingle() const throw () { return mult_lower == 1 && mult_upper == 1; }
	bool isMultUnlimited() const throw () { return mult_upper == -1; }
	bool isAtMostOne() const throw () { return mult_upper == 1; }
	bool isAtLeastOne() const throw () { return mult_lower >= 1; }
	int getMultLower() const throw () { return mult_lower; }
	int getMultUpper() const throw () { return mult_upper; }
	DELEGATE_FUNC(bool, isOrdered, const throw (), kind);
	DELEGATE_FUNC(bool, isBag, const throw (), kind);
	DELEGATE_FUNC(bool, isBidirectional, const throw (), kind);
	DELEGATE_FUNC(bool, isComposition, const throw (), kind);
	DELEGATE_FUNC(bool, isAssociation, const throw (), kind);
	DELEGATE_FUNC(bool, isUnserializable, const throw (), kind);
	DELEGATE_FUNC(bool, isAnonymous, const throw (), kind);

#ifndef SWIG
	kind_t getKindT() const throw() { return kind; }
	 void __makeAnonymousProperty();
	/** New Slot for e, based on this Property */
	Slot* newSlot(Element* e) throw (std::bad_alloc);
#endif
	const char* getName() const throw() {
		return this->key.c_str();
	}
	AbstractElementDefinition* getTarget() const throw () {
		return this->element_kind;
	}
	Property* getOppositeProperty() const throw () {
		return this->other;
	}

#ifdef SWIG
	Property(const char* _key, AbstractElementDefinition* element_kind, const char* other_key,
		 int multiplicity_lower, int multiplicity_upper,
		 int kind,
		 const std::string& default_value = "") {
		__init(_key, element_kind, other_key, multiplicity_lower,
		       multiplicity_upper, kind, default_value);
	}
#else
	Property(const char* _key, AbstractElementDefinition* elem_kind,
		 const char* other_key,
		 int multiplicity_lower, int multiplicity_upper,
		 kind_t k,
		 const std::string& default_value = ""
	     );
#endif
};

class EXPORT_coral Package : public AbstractElementDefinition {
public:	
	typedef autounreflist<AbstractElementDefinition*> elements_type;
private:
	std::string xmlns;
	elements_type imported_elements;
	elements_type elements;
public:
	const std::string getXMLNS() const { return this->xmlns; }
	void setXMLNS(const std::string& newxmlns);

	PythonSequenceIteratorDirect<coral::Package::elements_type::iterator,
	     coral::AbstractElementDefinition*>* __elements__() {
	     return new PythonSequenceIteratorDirect<
		  coral::Package::elements_type::iterator,
		  coral::AbstractElementDefinition*>(
		       this,
		       this->elements.begin(),
		       this->elements.end()
		       );
	}
	PythonSequenceIteratorDirect<coral::Package::elements_type::iterator,
	     coral::AbstractElementDefinition*>* __imported_elements__() {
	     return new PythonSequenceIteratorDirect<
		  coral::Package::elements_type::iterator,
		  coral::AbstractElementDefinition*>(
		       this,
		       this->imported_elements.begin(),
		       this->imported_elements.end()
		       );
	}


	Language* __language__() const throw();
	Package(Package* parent, const std::string& name);
	virtual ~Package() throw ();
	virtual coral::autounref<coral::AbstractElementDefinition>
	     getElement(const char* name) const throw (std::exception);
	coral::autounref<coral::AbstractElementDefinition> __getattr__(const char* name)
	     const throw (std::exception) {
	     return getElement(name);
	}
	void addElement(AbstractElementDefinition* mm);
	void removeElement(AbstractElementDefinition* mm);
	void importElement(AbstractElementDefinition* mm);
#ifndef SWIG
	virtual Atom* newAtom();
	virtual std::string __str__() const;
#endif
	static coral::autounref<coral::Package>
	     __getLanguageByXMLNS__(const std::string& xmlns,
				    bool allowdynamicloading = true)
	     throw (coral::language_not_found_error, std::exception);
 protected:
	static  coral::autounref<Language> tryDynLoad(const std::string& smd_model_filename);
};

#ifdef COMPILING_coral
	/* only Coral needs to have these */

	/* Protects all_languages. */
	EXTERN_MUTEX(mutex_languages);
#endif

class EXPORT_coral Language : public Package {
private:
	int version;
	int revision;
	ElementDefinition* root_ed;
 public:
	Language(const std::string& name,
		 int version, int revision);
	virtual ~Language() throw ();
	void setRoot(ElementDefinition* e) throw ();
	ElementDefinition* getRoot() const throw ();
	const int getVersion() const throw() { return this->version; }
	const int getRevision() const throw() { return this->revision; }

	static coral::autounref<coral::Language>
	     __getLanguage__(const std::string& name,
			     int ver, int rev,
			     bool allowdynamicloading = true)
	     throw (coral::language_not_found_error, std::exception);
#ifndef SWIG
	virtual std::string __str__() const;
#endif
};


}

#endif
#ifndef H_CORAL_OCL
#define H_CORAL_OCL
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.






namespace coral {
	namespace Transaction {
		class EXPORTFWD_coral Manager;
	}

     class EXPORTFWD_coral Iterator;
     class EXPORTFWD_coral ref_counted;
     class EXPORTFWD_coral Slot;
     class EXPORTFWD_coral Set;

     class EXPORT_coral Collection : public ref_counted, public noncopy {
     protected:
	  size_t n_elements;
	   Collection();
     public:
	  virtual ~Collection() throw ();
	  virtual bool isOrdered() const throw () { return false; }
	  virtual bool isBag() const throw () { return false; }
	  /* OCL compat. */
	  size_t size() const throw () { return this->n_elements; }
	  size_t __len__() const throw () { return this->n_elements; }
	  bool isEmpty() const throw () { return this->n_elements == 0; }
	  bool __nonzero__() const throw () { return this->n_elements != 0; }
	  bool nonEmpty() const throw () { return this->n_elements != 0; }
	  virtual bool includes(const ref_counted*) const throw () ;
	  bool __contains__(const ref_counted* e) const throw () {
		  return includes(e);
	  }
	  bool excludes(const ref_counted* e) const throw () { return (!this->includes(e)); }
	  virtual std::size_t count(const ref_counted*) const throw () ;
	  virtual bool includesAll(const Collection*) const throw ();
	  virtual bool excludesAll(const Collection*) const throw ();
	  virtual void unionWith(const Collection*) throw (std::runtime_error, std::exception);
	  virtual void intersectWith(const Collection*) throw (std::runtime_error, std::exception);
	  virtual coral::autounref<coral::Set> setDifference(const Collection*) const throw (std::runtime_error, std::exception);
	  /* select is in kernel.py */
	  /* reject is in kernel.py */
	  Iterator* __iter__() const throw (std::bad_alloc);
	  /*
	   * Returns true if we actually did insert something.
	   * Needed for Sets.
	   */
	  virtual bool insert(const ref_counted*, int index = -1) ;
	  virtual ref_counted* __getitem__(int index) const throw (std::out_of_range) ;
	  virtual bool __setitem__(int index, const ref_counted*) ;
	  virtual ref_counted* remove_keepref(int index) throw (std::out_of_range)  ;
	  virtual int drop(const ref_counted*) throw () ;

	  bool add(const ref_counted* e) { return this->insert(e, -1); }
	  bool append(const ref_counted* e) { return this->insert(e, -1); }
	  void drop_index(int index);

	  virtual void reset() throw () ;
	  coral::unicodestring __str__() const;
	  coral::unicodestring __unicode__() const { return __str__(); }
	  coral::autounref<coral::Collection> newSimilarCollection() const;
	  virtual coral::autounref<coral::Collection> clone() const;

	  static coral::autounref<coral::Collection> newCollection(bool is_bag, bool is_ordered);
     };

     class EXPORT_coral Bag: public Collection {
     public:
	  typedef __gnu_cxx::hash_multiset<const coral::ref_counted*, coral::ptr_hash<const coral::ref_counted> > values_type;
     private:
#ifndef SWIG
	  friend class Iterator;
	  DELEGATE_FUNC(values_type::iterator, begin,, values);
	  DELEGATE_FUNC(values_type::iterator, end,, values);
	  DELEGATE_FUNC(values_type::const_iterator, begin, const, values);
	  DELEGATE_FUNC(values_type::const_iterator, end, const, values);
	  DELEGATE_TYPE(values_type, iterator, iterator);
	  DELEGATE_TYPE(values_type, const_iterator, const_iterator);
#endif
	  values_type values;
     public:
	  Bag();
	  virtual ~Bag() throw ();
#ifndef SWIG
	  bool isBag() const throw (){ return true; }
	  bool insert(const ref_counted*, int index = -1);
	  ref_counted* __getitem__(int index) const throw (std::out_of_range);
	  bool __setitem__(int index, const ref_counted*);
	  ref_counted* remove_keepref(int index) throw (std::out_of_range);
	  int drop(const ref_counted*) throw ();

	  std::size_t count(const ref_counted*) const throw ();
	  void reset() throw ();
	  bool includes(const ref_counted*) const throw ();
#endif
     };

     class EXPORT_coral OrderedSet: public Collection {
     public:
	  /* Note that this is not a vector! Makes pushing/poping
	     very fast, but indexing slow. This might be very stupid also. */
	  typedef std::list<const ref_counted*> values_type;
     private:
#ifndef SWIG
	  friend class Iterator;
	  friend class SlotManyref_countedsSequence;
	  DELEGATE_FUNC(values_type::iterator, begin,, values);
	  DELEGATE_FUNC(values_type::iterator, end,, values);
	  DELEGATE_FUNC(values_type::const_iterator, begin, const, values);
	  DELEGATE_FUNC(values_type::const_iterator, end, const, values);
	  DELEGATE_TYPE(values_type, iterator, iterator);
	  DELEGATE_TYPE(values_type, const_iterator, const_iterator);
#endif
	  values_type values;
     public:
	  OrderedSet();
	  virtual ~OrderedSet() throw ();

#ifndef SWIG
	  bool isOrdered() const throw () { return true; }
	  ref_counted* __getitem__(int index) const throw (std::out_of_range);

	  bool insert(const ref_counted*, int index = -1);
	  bool __setitem__(int index, const ref_counted*);
	  ref_counted* remove_keepref(int index) throw (std::out_of_range);
	  int drop(const ref_counted*) throw ();

	  std::size_t count(const ref_counted*) const throw ();
	  void reset() throw ();
	  bool includes(const ref_counted*) const throw ();
#endif
     };

     class EXPORT_coral Sequence: public Collection {
     public:
	  /* Note that this is not a vector. */
	  typedef std::list<const ref_counted*> values_type;
     private:
#ifndef SWIG
	  friend class Iterator;
	  friend class SlotManyref_countedsSequence;
	  DELEGATE_FUNC(values_type::iterator, begin,, values);
	  DELEGATE_FUNC(values_type::iterator, end,, values);
	  DELEGATE_FUNC(values_type::const_iterator, begin, const, values);
	  DELEGATE_FUNC(values_type::const_iterator, end, const, values);
	  DELEGATE_TYPE(values_type, iterator, iterator);
	  DELEGATE_TYPE(values_type, const_iterator, const_iterator);
#endif
	  values_type values;
     public:
	  Sequence();
	  virtual ~Sequence() throw ();

#ifndef SWIG
	  bool isOrdered() const throw () { return true; }
	  bool isBag() const throw () { return true; }
	  ref_counted* __getitem__(int index) const throw (std::out_of_range);

	  bool insert(const ref_counted*, int index = -1);
	  bool __setitem__(int index, const ref_counted*);
	  ref_counted* remove_keepref(int index) throw (std::out_of_range);
	  int drop(const ref_counted*) throw ();

	  std::size_t count(const ref_counted*) const throw ();
	  void reset() throw ();
	  bool includes(const ref_counted*) const throw ();
#endif
     };


     class EXPORT_coral Set: public Collection {
     public:
	  typedef __gnu_cxx::hash_set<const coral::ref_counted*, coral::ptr_hash<coral::ref_counted> > values_type;
     private:
#ifndef SWIG
	  friend class Iterator;
	  friend class SlotManyref_countedsSet;
	  friend class Transaction::Manager;
	  DELEGATE_FUNC(values_type::iterator, begin,, values);
	  DELEGATE_FUNC(values_type::iterator, end,, values);
	  DELEGATE_FUNC(values_type::const_iterator, begin, const, values);
	  DELEGATE_FUNC(values_type::const_iterator, end, const, values);
	  DELEGATE_TYPE(values_type, iterator, iterator);
	  DELEGATE_TYPE(values_type, const_iterator, const_iterator);
#endif
	  values_type values;
     public:
	  Set();
	  virtual ~Set() throw ();
#ifndef SWIG
	  int drop(const ref_counted*) throw ();
	  /* HACK! */
	  ref_counted* __getitem__(int index) const throw (std::out_of_range);

	  bool includes(const ref_counted*) const throw ();
	  void reset() throw ();
	  bool insert(const ref_counted*, int index = -1);
	  bool __setitem__(int index, const ref_counted*);
	  ref_counted* remove_keepref(int index) throw (std::out_of_range);
	  std::size_t count(const ref_counted*) const throw ();
#endif
     };




     class EXPORT_coral Iterator : public ref_counted, public noncopy {
     private:
	  enum iter_type { ITER_MMSET,
			   ITER_MMBAG,
			   ITER_MMORDSET,
			   ITER_MMSEQ,
			   ITER_ONE_ELEMENT };
	  const Collection* collection;
	  enum iter_type iter_type;
	  Set::const_iterator seti;
	  Bag::const_iterator bagi;
	  OrderedSet::const_iterator oseti;
	  Sequence::const_iterator seqi;
	  const ref_counted* element;
	   void __collection(const Collection* c);
     public:
	  explicit Iterator(const Collection* c);
#ifndef SWIG
	  explicit Iterator(const Collection& c);
	  explicit Iterator(const Slot& s);
#endif
	  explicit Iterator(const Slot* c);
	  ~Iterator() throw ();
	  ref_counted* next() throw ();
	  Iterator* __iter__() throw () { return this; }
     };


}


#endif
#ifndef H_CORAL_PRIMITIVEPROPERTIES
#define H_CORAL_PRIMITIVEPROPERTIES
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.




namespace coral {

     class EXPORTFWD_coral Slot;
     class EXPORTFWD_coral Element;
     class EXPORTFWD_coral Atom;

     class EXPORT_coral DatatypeDefinition: public AbstractElementDefinition {
     public:
	  DatatypeDefinition(Package* metamodel, const char* name);
	  virtual ~DatatypeDefinition() throw ();
#ifndef SWIG
	  virtual bool isDatatypeDefinition() const throw () { return true; }
	  virtual Atom* newAtom();
	  virtual std::string __str__() const;
#endif
     };

     class EXPORT_coral EnumerationDefinition: public DatatypeDefinition {
     private:
	  std::vector<std::string> descriptions;
     public:
	  EnumerationDefinition(Package*, const char* name);
	  virtual ~EnumerationDefinition() throw ();
	  void __addEnum__(const char*) throw ();
	  std::string getDescription(int i) const throw ();
	  int getIndexByDescription(const char*) const throw ();
	  int size() const throw () { return this->descriptions.size(); }
#ifndef SWIG
	  virtual Atom* newAtom();
	  virtual bool isEnumerationDefinition() const throw () { return true; }
	  virtual std::string __str__() const;
#endif
     };

}

#endif

#ifndef H_CORAL_PYOPERATION
#define H_CORAL_PYOPERATION
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.



namespace coral {
     class EXPORTFWD_coral Element;

     class EXPORT_coral PyOperation : public ref_counted, public noncopy {
     private:
	  PyObject* function;
	  bool is_a_wfr;
	  std::string language;
	  std::string code;
     public:
	  PyOperation(PyObject* _function,
		      const std::string& _language,
		      const std::string& code,
		      bool _isWFR);
	  virtual ~PyOperation() throw ();
	  PyObject* getFunc() const throw ();
	  bool isWFR() const throw () { return this->is_a_wfr; }
	  std::string getFuncName() const throw ();
	  const std::string& getCode() const throw ();
	  PyObject* __call__(coral::Element* self);
	  const std::string& getOperationLanguage() const throw () { return this->language; }
     };
}

#endif

#ifndef H_CORAL_STORAGE
#define H_CORAL_STORAGE
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.


/*
 * For stat
 */

/*
 * Project is a mechanism to decide from which file a (root) element
 * came from. It's set for a root element in io_xmi_load_file
 *
 * Technically we ought to be able to use this (and a hacked Element
 * class) to implement load-on-demand models. This is N/A for now, though.
 *
 */
namespace coral {
     class EXPORTFWD_coral Element;

     class EXPORT_coral Project : public ref_counted {
     public:
	  typedef autounrefmap<const coral::io::_ref*,
		  coral::Element*,
		  coral::io::lt_std_ref>
		  old_loaded_ref_info_t;
     private:
	  /*
	   * True if storage is invalid. Used by copy/paste
	   * algorithms in the modeler.
	   */
	  bool m_isinvalid;
	  /*
	   * True if the project is a user project.
           */
	  bool m_isUserProject;
	  /*
	   * The true filename of this Project
	   */
	  std::string m_filename;
	  /* this is different from the "real" filename,
	   * it shows up in XMI files.
	   */
	  std::string m_hreffilename;
	  /*
	   * XMI format used previously. Empty if undefined.
	   */
	  std::string m_xmiformat;
	  // Ordered for practical purposes
	  Sequence* m_roots;
	  /*
	   * The old xmi:uuid etc data... that we got when loading
	   */
	  old_loaded_ref_info_t m_old_loaded_ref_info;
	  /*
	   * For making sure two Project objects never point
	   * to the same file.
	   */
	  dev_t m_device;
	  ino_t m_inode;

	   void __setFilename_nocheck(const std::string& f);
	   void __setHrefFilename(const std::string& f);
     public:
#ifndef SWIG
	   Element* findRef(const coral::io::_ref*);
	   old_loaded_ref_info_t& getOldInfo() throw () {
		  return m_old_loaded_ref_info;
	  }
	   void fixLoadedRef();
#endif
	  Project();
	  ~Project() throw ();
	  std::string __str__() const;
	  bool isInvalid() const throw () { return m_isinvalid; }
	  bool isUserProject() const throw () { return m_isUserProject; }
	  void setUserProject(bool b) { m_isUserProject=b;}
	  void invalidate();
	    // @deprecated
	  void setInvalid(bool b) { coral_assert(b); invalidate(); }
	  std::string getXMIFormat() const { return m_xmiformat; }
	  void setXMIFormat(const std::string& s) { m_xmiformat = s; }
	  Sequence* contents() const throw () { return m_roots; }
	  void setFilename(const std::string& f);
	  const std::string& getFilename() const throw (){ return m_filename; }
	  const std::string& getHrefFilename() const throw () {
		  return m_hreffilename;
	  }
#ifdef COMPILING_coral
	  // use element.__getProject__ or __setProject__ instead of these
	  static Project* getProject(const Element*);
	  static void setProject(const Element*, Project*);
#endif
	  /* Return a reffed Project object for a particular filename. */
	  static coral::autounref<coral::Project>
	       getProjectByFilename(const std::string& filename,
				    const std::string& relative_path = "",
				    bool* did_create = NULL);
     };

     /**
      * Returns a vector of all active Projects, unreffed
      */
     std::vector<coral::Project*> getAllProjects();

     std::string getCanonicalFilename(const std::string& input);

     /*
      * Get the search path of Coral. By default, we only
      * search the current directory (relative to some other file),
      * the startup directory, and Coral's installation path.
      *
      */
     std::vector<std::string> getCoralPath();

     void setCoralPath(std::vector<std::string> s);

}


#endif
#ifndef H_CORAL_XMLNS_CONVERSION_TABLE
#define H_CORAL_XMLNS_CONVERSION_TABLE
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.


/*
 * Conversion table for supporting mapping metamodel URIs to
 * the actual metamodel.
 *
 */

namespace coral {

#ifndef SWIG
	struct xmlns_conversion_entry {
		std::string xmlns;
		std::string smd_model_filename;
		std::string metamodelname;
		int version;
		int revision;
		xmlns_conversion_entry() { ; }
		~xmlns_conversion_entry() { ; }
		bool operator==(const struct xmlns_conversion_entry& t) const {
			return xmlns == t.xmlns
				&& smd_model_filename == t.smd_model_filename
				&& metamodelname == t.metamodelname
				&& version == t.version
				&& revision == t.revision;
		}
		bool operator!=(const struct xmlns_conversion_entry& t) const {
			return !(*this == t);
		}
	};

	/*
	 * Get a reverse map entry. The xmlns field
	 * must be filled in, others are discarded.
	 * Returns true on success, otherwise false
	 * and no fields are touched.
	 */
	bool getXMLNSRmap(struct xmlns_conversion_entry* fill_in);

#endif
	/*
	 * Add a reverse map entry from XML namespace string
	 * to metamodel name, version and revision
	 * e.g. http://schema.omg.org/spec/UML/1.4 => UML, 1, 4
	 * Returns true on success. Multiple insertion of same
	 * data allowed.
	 */
	bool addXMLNSRmap(const std::string& xmlns,
			  const std::string& filename,
			  const std::string& metamodelname,
			  int version,
			  int revision);

	/*
	 * Returns the size of the table
	 * Immediately afterwards, without any other xmlns operations
	 * the various xmlns_db_* functions are valid and available.
	 */
	int xmlns_db_size();
	std::string xmlns_db_xmlns(int idx);
	std::string xmlns_db_filename(int idx);
	std::string xmlns_db_metamodelname(int idx);
	int xmlns_db_version(int idx);
	int xmlns_db_revision(int idx);
}

#endif
#ifndef H_CORAL_GARBAGE_COLLECTOR
#define H_CORAL_GARBAGE_COLLECTOR
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.



namespace coral {

	/**
	 * Garbage collect elements that are not in use.
	 * Warning! This might take a long time.
	 */
	void EXPORT_coral garbage_collect() throw ();
#ifdef COMPILING_coral
	// Don't use this...
	void x__garbage_collect();
#endif
}


#endif
#ifndef H_CORAL_MODELCOPY
#define H_CORAL_MODELCOPY
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.

/*
 * Implements the modelcopy algorithm described in
 * TUCS Technical Report 486
 *
 */


namespace coral {

	/*
	 * Modification bits for modelcopy functionality
	 * The default is the _opposite_ of the list below.
	 */
#ifndef SWIG
	enum modelcopy_params {
		MODELCOPY_DONT_CREATE_NEW_UUIDS = 1,
		MODELCOPY_DONT_ATTACH_TO_PARENTS = 2,
		MODELCOPY_DONT_SKIP_PYOBJECTS = 4,
		MODELCOPY_DONT_KEEP_ASSOCIATIONS_OUTSIDE_SUBTREE = 8,
	};
#endif

	class EXPORTFWD_coral Collection;
	class EXPORTFWD_coral Set;
	class EXPORTFWD_coral Sequence;
	class EXPORTFWD_coral Atom;
	class EXPORTFWD_coral Element;

	typedef std::map<const coral::Element*, 
	     const coral::Element*> modelcopy_map_t;

	/*
	 * The modelcopy routines fill the two maps
	 * original_to_copy and copy_to_original.
	 * A copy C of an element O has entries
	 * original_to_copy[O] == C and
	 * copy_to_original[C] == O.
	 */

	EXPORT_coral
	     coral::autounref<coral::Sequence>
	     ModelCopy(const coral::Collection*,
		       int params = 0,
		       modelcopy_map_t* original_to_copy = NULL,
		       modelcopy_map_t* copy_to_original = NULL);
	
	EXPORT_coral
	     coral::autounref<coral::Atom>
	     ModelCopy_One(const coral::Atom*,
			   int params = 0,
			   modelcopy_map_t* original_to_copy = NULL,
			   modelcopy_map_t* copy_to_original = NULL);
}


#endif
#ifndef H_CORAL_SAMEMODEL
#define H_CORAL_SAMEMODEL
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.


namespace coral {
	class EXPORTFWD_coral Atom;
	EXPORT_coral bool isSameModel(const coral::Atom* a1,
				      const coral::Atom* a2);
}

#endif
#ifndef H_CORAL_CORE_UTIL_UELEMENT
#define H_CORAL_CORE_UTIL_UELEMENT
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.

/*
 * Helper functions for the Element class, separated
 * here because they don't need to muck with the inner workings
 * of Element.
 */


namespace coral {
namespace util {

/*
 * Returns itself and the list of parents
 * beginning from itself traversing upwards.
 */
coral::autounref<coral::OrderedSet>
Element__getMyselfAndParents__(const Element* e, bool includemyself = true);


/*
 * Returns the list of parents and itself
 * beginning from the topmost traversing downwards.
 */
coral::autounref<coral::OrderedSet>
Element__getParentsAndMyself__(const Element* e, bool includemyself = true);


/*
 * Maps x,y in XMI-DI GraphElement coordinates (origin at top-left of
 * the GraphElement) to global coordinates (origin at top-left of world)
 */
std::pair<double, double> XMIDI_LocalToGlobalPosition(const Element* e,
						      double x,
						      double y);

/*
 * Maps x,y in global coordinates (origin at top-left of world) to 
 * XMI-DI GraphElement coordinates (origin at top-left of the GraphElement) 
 */
std::pair<double, double> XMIDI_GlobalToLocalPosition(const Element* e,
						      double x,
						      double y);

/* Returns true if DiagramElement de's
 * semanticBridge is referential. This means that next container of
 * de references a semantic element that is not the parent of de's
 * referenced semantic element.
 *
 * i.e. in this case:
 * 
 *  parent  <-------- container
 *    |                  |
 *  child   <-------- contained (de)
 * 
 * Other situations return false.
 */

bool XMIDI_HasReferentialRole(const Element* de);

coral::Element*
XMIDI_find_property_upwards(coral::Element* e,
			    const std::string& propname);

coral::Element*
XMIDI_find_either_property_upwards(coral::Element* e,
				   const std::string& propname1,
				   const std::string& propname2);

#ifndef SWIG
/**
 * Returns name of element if it exists and is suitable
 */
const ElementString* Element__getName(const Element* e);
#endif

}
}

#endif
#ifndef H_CORAL_DIFF_DIFFERENCE
#define H_CORAL_DIFF_DIFFERENCE
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.



namespace coral {
namespace DIFF {

#ifndef SWIG
enum modeldifference_params {
  MODELDIFFERENCE_IGNORE_DIAGRAM_INFORMATION = 1,
  MODELDIFFERENCE_ONLY_DIAGRAM_INFORMATION = 2,
};
#endif

#ifndef SWIG

typedef auto2unrefmap<const char*, const Element*, less_strcmp<const char*> >
model_elements_t;
void getUUID2ElementMap(const Collection* roots, model_elements_t& mret);

#endif


/**
 * Compares two collections and returns a DIFF10 model
 *
 */
coral::autounref<coral::Element> EXPORT_coral
ModelDifference(const Collection* roots1,
		const Collection* roots2,
		int params = 0) throw (std::exception);

coral::autounref<coral::Element>
EXPORT_coral
ModelDifference_One(const Element* root1,
		    const Element* root2,
		    int params = 0) throw (std::exception);


}
}


#endif
#ifndef H_coral_CORE
#define H_coral_CORE
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.


namespace coral {
namespace CORE {

#ifndef SWIG

extern EXPORT_coral coral::DatatypeDefinition* Name;
extern EXPORT_coral coral::DatatypeDefinition* String;
extern EXPORT_coral coral::EnumerationDefinition* Boolean;
extern EXPORT_coral coral::DatatypeDefinition* Integer;
extern EXPORT_coral coral::DatatypeDefinition* UnlimitedInteger;
extern EXPORT_coral coral::DatatypeDefinition* UnlimitedReal;
extern EXPORT_coral coral::DatatypeDefinition* Float;
extern EXPORT_coral coral::DatatypeDefinition* Double;
extern EXPORT_coral coral::DatatypeDefinition* coralPyObj;

extern EXPORT_coral coral::Language* language;
extern EXPORT_coral coral::ElementDefinition* AnyElement;
extern EXPORT_coral coral::ElementDefinition* AnyAbstractElement;
extern EXPORT_coral coral::ElementDefinition* PresentationElement;

#endif

#ifdef COMPILING_coral
extern  bool init_language() throw ();
#endif

}
}

#endif

#ifndef H_CORAL_LANG_SMD01_SMDLANG
#define H_CORAL_LANG_SMD01_SMDLANG
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.


namespace coral {

class Meta2ModelBridge;
class Element;
class Language;

namespace SMD01 {

/*
 * Maps an SMD model to Coral data structures
 * This is the interface for creating new
 * metamodels out of SMD models.
 *
 * You may pass NULL, in which case you get NULL,
 * otherwise a new complete Coral language is returned.
 */
coral::autounref<coral::Language> smd2lang(coral::Element* language) throw (std::exception);

}
}

#endif

#ifndef H_CORAL_LIB_DEBUGSTREAM
#define H_CORAL_LIB_DEBUGSTREAM
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.



namespace coral {

#ifndef SWIG
enum debuglevel {
     DEBUGLEVEL_ALL,
     DEBUGLEVEL_DEBUG,
     DEBUGLEVEL_INFO,
     DEBUGLEVEL_WARNING,
     DEBUGLEVEL_ERROR,
     DEBUGLEVEL_CRITICAL,
     DEBUGLEVEL_NONE,
};
#endif

#ifndef SWIG

extern EXPORT_coral const char* debuglevel_str[];

/*
 * Debug streambuf which outputs to given output stream as well
 * as to an internal stringstream buffer
 *
 */
class EXPORT_coral debugstreambuf
: public std::basic_streambuf<char, std::char_traits<char> > {
private:
     std::ostringstream m_output1;
     std::ostream& m_output2;
     debuglevel current_message_dlevel;
public:
     debugstreambuf(std::ostream& out);
     ~debugstreambuf();
     debuglevel& curMessageLevel() { return current_message_dlevel; }
     std::streamsize xsputn(const char* s, std::streamsize n);
     int overflow(int c = traits_type::eof());
     std::ostringstream& get_string_stream();
};

/*
 * The log output stream, which writes to std::clog and the dlogbuf
 */
extern EXPORT_coral std::ostream dlog;

extern EXPORT_coral std::ostream& operator<<(std::ostream& out, debuglevel d);

#endif


#ifndef SWIG
/*
 * Sets debuglevel, returns old.
 */
debuglevel setDebugLevel(debuglevel d);
/*
 * Returns debuglevel
 */
debuglevel getDebugLevel();
#endif

std::string getDebugLog();

void setDebugObserver(PyObject*);

}

#endif
#ifndef H_CORAL_UUID
#define H_CORAL_UUID
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.


namespace coral {

	typedef std::string uuid_t;

	uuid_t EXPORT_coral getUniqueID();

#ifdef COMPILING_coral
	EXTERN_MUTEX(mutex_uuid);
#endif


#ifndef SWIG
#ifdef __MINGW32__
	int EXPORT_coral gettimeofday(struct timeval* p,
				      struct timezone* tz /* IGNORED */);
#endif

	void initGetUniqueID();
#endif

}
#endif
#ifndef H_CORAL_TRANSACTIONMANAGER
#define H_CORAL_TRANSACTIONMANAGER
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.




namespace coral {
namespace Transaction {

class EXPORTFWD_coral Manager;
class EXPORTFWD_coral Transaction;
class EXPORTFWD_coral Command;
#ifndef SWIG
class PauseTransaction;
extern  Manager* theTM;

#endif

/*
 * These are attached to the Manager.
 * One per transaction.
 */
class EXPORT_coral Transaction
: public ref_counted, protected List {
private:
     friend class Manager;
     /* These are the individual commands. */
     typedef autounrefvector<Command*> commands_type;
     commands_type m_commands;

public:
     Transaction();
     ~Transaction() throw ();
     std::string __str__() const;
     size_t size() const throw ();
#ifndef SWIG
     Command* operator[](int pos);
#endif
     Command* __getitem__(int pos) {
	  return (*this)[pos];
     }

};

#ifndef SWIG
/**
 * Forgets the previous transaction command upon destruction.
 * Can be overridden by calling release().
 */
class EXPORT_coral ForgetPreviousCommand {
	bool m_destroy;
public:
	ForgetPreviousCommand() :
		m_destroy(true) {
		;
	}
	~ForgetPreviousCommand() throw ();
	void release() { m_destroy = false; }
};

#endif

/*
 * The individual commands in a
 * Transaction
 *
 */
class EXPORT_coral Command : public ref_counted, public noncopy {
protected:
     Command() { };
public:
     virtual ~Command() throw ();
     virtual std::string __str__() const ;
     virtual void execute() throw ();
     virtual void unexecute() throw () ;
};

class EXPORT_coral ElementCommand : public Command {
protected:
     Atom* e;
     ElementCommand(Atom* elem ) {
	  coral_assert(elem); this->e = elem; elem->ref();
     }
public:
     virtual ~ElementCommand() throw () { this->e->unref(); }
     coral::Atom* getAtom() const throw () {
	  return this->e;
     }
};

class EXPORT_coral NewProjectCommand : public Command {
protected:
	Project* m_newstorage;
public:
	NewProjectCommand(Project* s) : m_newstorage(s) {
		coral_assert(s); s->ref();
	}
	~NewProjectCommand() throw () { m_newstorage->unref(); } 
	Project* getNewProject() const throw () { return m_newstorage; }
#ifndef SWIG
	std::string __str__() const;
	void unexecute() throw () { ; }
#endif
};

class EXPORT_coral ProjectNameChangeCommand : public Command {
protected:
	Project* m_storage;
	std::string m_old_filename;
	std::string m_old_href_name;
public:
	ProjectNameChangeCommand(Project* s,
				 const std::string& old_filename,
				 const std::string& old_href_name);
	~ProjectNameChangeCommand() throw () { m_storage->unref(); } 
	Project* getProject() const throw () { return m_storage; }
	const std::string& getOldFilename() const throw () { return m_old_filename; }
	const std::string& getOldHrefFilename() const throw () { return m_old_href_name; }
#ifndef SWIG
	std::string __str__() const;
	void unexecute() throw ();
#endif
};


class EXPORT_coral ProjectChangeCommand : public ElementCommand {
protected:
     Project* m_old;
public:
     ProjectChangeCommand(Element* e, Project* old);
     ~ProjectChangeCommand() throw ();
     Project* getOldProject() const throw () { return m_old; }
#ifndef SWIG
     std::string __str__() const;
     void unexecute() throw ();
#endif
};

class EXPORT_coral ChangeElementPrimitiveCommand : public ElementCommand {
private:
     ElementPrimitive* new_atom;
public:
     ChangeElementPrimitiveCommand(coral::ElementPrimitive* elem)
	  : ElementCommand(elem->clone()) {
	  new_atom = elem;
	  new_atom->ref();
     }
	  virtual ~ChangeElementPrimitiveCommand() throw () {
	       this->new_atom->unref();
	  }
	  coral::ElementPrimitive* getNewAtom() const throw () {
	       return this->new_atom;
	  }
#ifndef SWIG
	  std::string __str__() const;
	  void unexecute() throw ();
#endif
};

class EXPORT_coral ElementChangeSlotCommand : public ElementCommand {
private:
     std::auto_ptr<coral::Slot> previous;
public:
     ElementChangeSlotCommand(Element* elem, Slot* theprevious)
	  : ElementCommand(elem),
	  previous(theprevious->clone()) { ; }
	  virtual ~ElementChangeSlotCommand() throw () { ; }
	  coral::Slot* getSlot() const throw () {
	       return this->previous.get();
	  }
#ifndef SWIG
	  std::string __str__() const;
	  void unexecute() throw ();
#endif
};

class EXPORT_coral ElementNewCommand : public ElementCommand {
public:
     ElementNewCommand(coral::Element* elem)
	  : ElementCommand(elem) { ; }
	  virtual ~ElementNewCommand() throw () { ; }
	  coral::Element* getNewElement() const throw () {
	       return static_cast<coral::Element*>(this->e);
	  }
#ifndef SWIG
	  std::string __str__() const;
	  void unexecute() throw ();
#endif
};

class EXPORT_coral ElementUUIDCommand : public ElementCommand {
private:
     std::string XMIid;
public:
     ElementUUIDCommand(coral::Element* elem,
		 const std::string& xmiid) 
	  : ElementCommand(elem), XMIid(xmiid) { ; }
	  virtual ~ElementUUIDCommand() throw () { ; }
	  std::string getOldXMIid() const throw () {
	       return this->XMIid;
	  }
#ifndef SWIG
	  std::string __str__() const;
	  void unexecute() throw ();
#endif
};

/*
 * The Transaction Manager.
 */
extern EXPORT_coral Manager* getTransactionManager();

class EXPORT_coral Manager {
private:
	enum transaction_type { TRANSACTION_REDO,
				TRANSACTION_UNDO,
				TRANSACTION_ORDINARY };
     typedef std::list<PyObject*> observers_type;
     observers_type  m_observers;
     typedef std::list<PyObject*> busyObservers_type;
     busyObservers_type  m_busyObservers;
     bool m_active;
     bool m_strict;
     int m_n_transactions;
     /* All items in this manager have this as
      * their base. */
     List m_transactions;
     /* beginModifyCmd recursion depth. */
     int m_inModifyCmd;
     /* We have executed this command, we're going
      * to move to the next.
      * So at start, currentCmd == transactions
      */
     Transaction* m_currentCmd;
     transaction_type m_current_tr_in_progress;
#ifndef SWIG			
     friend Manager* getTransactionManager();
     friend class PauseTransaction;
#endif
      Manager();
     /**
      * Destroys elements that should be deleted
      */
      void orphanManager();
     /**
      * Calls python observers.
      */
      void callObservers(transaction_type forwards);
      void callBusyObservers(bool isBusy);
     int m_amount_of_items_in_current_trans;
      void trimPast_w_history(int limit) throw ();
     int m_history_limit;

     /* Remember which Element-Property pairs have been
      * altered, and don't issue extra Commands to the Transaction.
      * This saves space and processing time. */
     typedef std::pair<const coral::Element*, const coral::Property*> content_t;
     typedef std::set<content_t> watch_t;
     watch_t m_watch;
public:
      ~Manager() throw ();
      bool already_modified(const coral::Element*,
					const coral::Property*) const;
#ifndef SWIG
     /**
      * Is the Transaction Manager active at the moment and without pauses?
      * This is mostly used internally by the core.
      */
      bool internalIsActive() throw ();
#endif
     /**
      * Are we currently in an undo or a redo phase, instead of
      * an ordinary transaction?
      */
     bool isUndoOrRedo() throw ();
     /**
      * Observers are called at the end of each
      * transaction, in the order they were added.
      */
     void addObserver(PyObject*);
     void addBusyObserver(PyObject*);
     /**
      * Remove an observer.
      */
     void removeBusyObserver(PyObject*);
     void removeObserver(PyObject*);

     /**
      * Amount of Transactions in history
      */
     int getAmountOfTransactions() const throw ();
     /*
      * What is our current modification?
      * 0 = first
      * ...
      * ...
      * amountOfModications-1 = last
      * amountOfModifications = invalid
      */
     int getCurrentTransactionPosition() const throw ();
     Transaction* getTransaction(int pos) throw ();
     /**
      * Returns the current transaction, whether we're
      * in an ordinary, undo, or redo phase.
      */
     Transaction* getCurrentTransaction() const throw ();

     /**
      * Starts a new transaction, or continues an old one.
      */
     void beginModifyCmd(bool strict = false) throw ();
#ifndef SWIG
     void partialModifyCmd(Command* c) throw ();
      void __forget_previous_Cmd() throw ();
#endif
     /**
      * Ends a transaction. If it's the last one matching
      * a beginModifyCmd(), the observers are run.
      */
     bool endModifyCmd() throw (std::exception);
     /* same as beginModifyCmd */
     void beginTransaction(bool strict = false) throw ();
     bool endTransaction() throw ();
     void beginStrictTransaction() throw () {
	  this->beginTransaction(true);
     }
     bool endStrictTransaction() throw () {
	  return this->endTransaction();
     }
     /*
      * Returns true if we're in the middle of a transaction
      */
     bool isCreatingTransaction() throw ();

     /**
      * Returns the history limit, i.e. the amount of
      * transactions that we remember and thus can undo/redo.
      */
     bool getHistoryLimit() throw () {
	  return m_history_limit;
     }
     /**
      * Sets the history limit.
      */
     void setHistoryLimit(int limit ) throw () {
	  m_history_limit = limit;
     }
     void trimPast() throw () {
	  trimPast_w_history(0);
     }
     void trimFuture() throw ();
     void clear() throw ();
     /**
      * Returns true if we can undo now
      */
     bool canUndo() throw ();
     /**
      * Returns true if we can redo now
      */
     bool canRedo() throw ();
     /**
      * Perform an undo. The individual commands in the current
      * transaction are run, and the observers are run.
      * NOTE! The observers may not change the model in _any_ way.
      */
     void undo() throw ();
     /**
      * Perform an redo. The individual commands in the current
      * transaction are run, and the observers are run.
      * NOTE! The observers may not change the model in _any_ way.
      */
     void redo() throw ();
};

#ifndef SWIG

class MakeTransaction : public noncopy {
public:
     MakeTransaction();
     ~MakeTransaction();
};

#endif

#ifdef COMPILING_coral
#ifndef SWIG
/*
 * Use to conditionally pause transactions. See Model.cpp.
 * Only used internally.
 * Only usable stack allocated, not heap allocated.
 */
class PauseTransaction : public noncopy {
private:
     bool m_pause;
public:
     PauseTransaction(bool yes = true);
     ~PauseTransaction() throw ();
     void setPause(bool yes) throw ();
};
#endif
#endif
}
}


#endif
#ifndef H_CORAL_IO
#define H_CORAL_IO
// This file is part of the Coral project.
// Please read the copyright notice in the LICENSE file.




namespace coral {
class EXPORTFWD_coral Element;
class EXPORTFWD_coral Language;
class EXPORTFWD_coral Project;
class EXPORTFWD_coral Collection;
	
namespace io {


/*
 * Deprecated. Use loadDocument/saveDocument instead
 */
EXPORT_coral
coral::Element*
loadModel(const char* filename,
	  coral::Language* l = NULL) throw (coral::load_error, std::exception);

EXPORT_coral
void
saveModel(const char* filename,
	  coral::Element* root,
	  const char* xmi_version = "1.2")
throw (coral::save_error, std::exception);
	     


EXPORT_coral
coral::autounref<coral::Project> EXPORT_coral
loadDocument(const char* filename,
	     coral::Language* l = NULL) throw (coral::load_error, std::exception);

EXPORT_coral
coral::autounref<coral::Project> EXPORT_coral
loadDocument_str(const char* buffer) throw (coral::load_error, std::exception);

/*
 * Saves a Project with its current filename and xmi format.
 */
EXPORT_coral void saveDocument(coral::Project* st) 
     throw (coral::save_error, std::exception);

#ifndef SWIG	     
EXPORT_coral void saveDocument_raw(std::ostream& out,
				   const coral::Collection* coll,
				   const char* xmi_version)
     throw (coral::save_error, std::exception);
#endif
	     
EXPORT_coral std::string saveDocument_str(const coral::Collection* coll,
					  const char* xmi_version)
     throw (coral::save_error, std::exception);
	     
std::vector<std::string> EXPORT_coral getSupportedXMIVersions();
	     
#ifndef SWIG
class ModelStreamer {
 public:
	virtual ~ModelStreamer() throw ();
	ModelStreamer();
	virtual void saveModel(const coral::Collection*,
			       std::ostream&) = 0;
};
#endif

}
}

#endif
